@@ -14,9 +14,6 @@
 /// `i686-pc-windows-msvc` target will have an `x86` module here, whereas
 /// `x86_64-pc-windows-msvc` has `x86_64`.
 ///
-/// > **Note**: This module is currently unstable. It was designed in
-/// > [RFC 2325][rfc] and is currently [tracked] for stabilization.
-///
 /// [rfc]: https://github.com/rust-lang/rfcs/pull/2325
 /// [tracked]: https://github.com/rust-lang/rust/issues/48556
 ///
@@ -61,8 +58,12 @@
 /// available, and can be used like so:
 ///
 /// ```ignore
-/// #[cfg(all(any(target_arch = "x86", target_arch = "x86_64"),
-///       target_feature = "avx2"))]
+/// #[cfg(
+///     all(
+///         any(target_arch = "x86", target_arch = "x86_64"),
+///         target_feature = "avx2"
+///     )
+/// )]
 /// fn foo() {
 ///     #[cfg(target_arch = "x86")]
 ///     use std::arch::x86::_mm256_add_epi64;
@@ -146,12 +147,10 @@
 /// expression evaluating to whether the local CPU has the AVX2 feature or
 /// not.
 ///
-///   Note that this macro, like the `arch` module, is platform-specific. The
-///   name of the macro is the same across platforms, but the arguments to the
-///   macro are only the features for the current platform. For example calling
-///   `is_x86_feature_detected!("avx2")` on ARM will be a compile time
-///   error. To ensure we don't hit this error a statement level `#[cfg]` is
-///   used to only compile usage of the macro on `x86`/`x86_64`.
+///   Note that this macro, like the `arch` module, is platform-specific. For
+///   example calling `is_x86_feature_detected!("avx2")` on ARM will be a
+///   compile time error. To ensure we don't hit this error a statement level
+///   `#[cfg]` is used to only compile usage of the macro on `x86`/`x86_64`.
 ///
 /// * Next up we see our AVX2-enabled function, `foo_avx2`. This function is
 ///   decorated with the `#[target_feature]` attribute which enables a CPU
@@ -188,11 +187,23 @@
 /// * [`x86_64`]
 /// * [`arm`]
 /// * [`aarch64`]
+/// * [`mips`]
+/// * [`mips64`]
+/// * [`powerpc`]
+/// * [`powerpc64`]
+/// * [`nvptx`]
+/// * [`wasm32`]
 ///
-/// [`x86`]: https://rust-lang-nursery.github.io/stdsimd/i686/stdsimd/arch/x86/index.html
-/// [`x86_64`]: https://rust-lang-nursery.github.io/stdsimd/x86_64/stdsimd/arch/x86_64/index.html
-/// [`arm`]: https://rust-lang-nursery.github.io/stdsimd/arm/stdsimd/arch/arm/index.html
-/// [`aarch64`]: https://rust-lang-nursery.github.io/stdsimd/aarch64/stdsimd/arch/aarch64/index.html
+/// [`x86`]: x86/index.html
+/// [`x86_64`]: x86_64/index.html
+/// [`arm`]: arm/index.html
+/// [`aarch64`]: aarch64/index.html
+/// [`mips`]: mips/index.html
+/// [`mips64`]: mips64/index.html
+/// [`powerpc`]: powerpc/index.html
+/// [`powerpc64`]: powerpc64/index.html
+/// [`nvptx`]: nvptx/index.html
+/// [`wasm32`]: wasm32/index.html
 ///
 /// # Examples
 ///
@@ -201,8 +212,7 @@
 /// AVX2 and also for the default platform.
 ///
 /// ```rust
-/// #![feature(cfg_target_feature, target_feature, stdsimd)]
-///
+/// # #![cfg_attr(not(dox),feature(cfg_target_feature, target_feature, stdsimd))]
 /// # #[cfg(not(dox))]
 /// # #[macro_use]
 /// # extern crate stdsimd;
@@ -219,7 +229,7 @@
 ///         // Note that this `unsafe` block is safe because we're testing
 ///         // that the `avx2` feature is indeed available on our CPU.
 ///         if is_x86_feature_detected!("avx2") {
-///             return unsafe { add_quickly_avx2(a, b, c) }
+///             return unsafe { add_quickly_avx2(a, b, c) };
 ///         }
 ///     }
 ///
@@ -243,7 +253,7 @@
 /// we'll be using SSE4.1 features to implement hex encoding.
 ///
 /// ```
-/// #![feature(cfg_target_feature, target_feature, stdsimd)]
+/// # #![cfg_attr(not(dox),feature(cfg_target_feature, target_feature, stdsimd))]
 /// # #![cfg_attr(not(dox), no_std)]
 /// # #[cfg(not(dox))]
 /// # extern crate std as real_std;
@@ -278,7 +288,8 @@
 ///     hex_encode_fallback(src, dst)
 /// }
 ///
-/// // translated from https://github.com/Matherunner/bin2hex-sse/blob/master/base16_sse4.cpp
+/// // translated from
+/// // https://github.com/Matherunner/bin2hex-sse/blob/master/base16_sse4.cpp
 /// #[target_feature(enable = "sse4.1")]
 /// #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
 /// unsafe fn hex_encode_sse41(mut src: &[u8], dst: &mut [u8]) {
@@ -318,7 +329,10 @@
 ///         let res2 = _mm_unpackhi_epi8(masked2, masked1);
 ///
 ///         _mm_storeu_si128(dst.as_mut_ptr().offset(i * 2) as *mut _, res1);
-///         _mm_storeu_si128(dst.as_mut_ptr().offset(i * 2 + 16) as *mut _, res2);
+///         _mm_storeu_si128(
+///             dst.as_mut_ptr().offset(i * 2 + 16) as *mut _,
+///             res2,
+///         );
 ///         src = &src[16..];
 ///         i += 16;
 ///     }
@@ -339,24 +353,50 @@
 ///     }
 /// }
 /// ```
-#[unstable(feature = "stdsimd", issue = "0")]
+#[stable(feature = "simd_arch", since = "1.27.0")]
 pub mod arch {
     #[cfg(all(not(dox), target_arch = "x86"))]
+    #[stable(feature = "simd_x86", since = "1.27.0")]
     pub use coresimd::arch::x86;
 
     #[cfg(all(not(dox), target_arch = "x86_64"))]
+    #[stable(feature = "simd_x86", since = "1.27.0")]
     pub use coresimd::arch::x86_64;
 
     #[cfg(all(not(dox), target_arch = "arm"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
     pub use coresimd::arch::arm;
 
     #[cfg(all(not(dox), target_arch = "aarch64"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
     pub use coresimd::arch::aarch64;
 
-    #[cfg(target_arch = "wasm32")]
+    #[cfg(all(not(dox), target_arch = "mips"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub use coresimd::arch::mips;
+
+    #[cfg(all(not(dox), target_arch = "mips64"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub use coresimd::arch::mips64;
+
+    #[cfg(all(not(dox), target_arch = "powerpc"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub use coresimd::arch::powerpc;
+
+    #[cfg(all(not(dox), target_arch = "powerpc64"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub use coresimd::arch::powerpc64;
+
+    #[cfg(all(not(dox), any(target_arch = "nvptx", target_arch = "nvptx64")))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub use coresimd::arch::nvptx;
+
+    #[cfg(all(not(dox), target_arch = "wasm32"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
     pub use coresimd::arch::wasm32;
 
     #[doc(hidden)] // unstable implementation detail
+    #[unstable(feature = "stdsimd", issue = "27731")]
     pub mod detect;
 
     /// Platform-specific intrinsics for the `x86` platform.
@@ -368,6 +408,7 @@ pub mod arch {
     /// [libcore]: ../../../core/arch/x86/index.html
     #[cfg(dox)]
     #[doc(cfg(target_arch = "x86"))]
+    #[stable(feature = "simd_x86", since = "1.27.0")]
     pub mod x86 {}
 
     /// Platform-specific intrinsics for the `x86_64` platform.
@@ -379,6 +420,7 @@ pub mod arch {
     /// [libcore]: ../../../core/arch/x86_64/index.html
     #[cfg(dox)]
     #[doc(cfg(target_arch = "x86_64"))]
+    #[stable(feature = "simd_x86", since = "1.27.0")]
     pub mod x86_64 {}
 
     /// Platform-specific intrinsics for the `arm` platform.
@@ -390,6 +432,7 @@ pub mod arch {
     /// [libcore]: ../../../core/arch/arm/index.html
     #[cfg(dox)]
     #[doc(cfg(target_arch = "arm"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
     pub mod arm {}
 
     /// Platform-specific intrinsics for the `aarch64` platform.
@@ -401,8 +444,78 @@ pub mod arch {
     /// [libcore]: ../../../core/arch/aarch64/index.html
     #[cfg(dox)]
     #[doc(cfg(target_arch = "aarch64"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
     pub mod aarch64 {}
-}
 
-#[unstable(feature = "stdsimd", issue = "0")]
-pub use coresimd::simd;
+    /// Platform-specific intrinsics for the `mips` platform.
+    ///
+    /// The documentation with the full listing of `mips` intrinsics is
+    /// available in [libcore], but the module is re-exported here in std
+    /// as well.
+    ///
+    /// [libcore]: ../../../core/arch/mips/index.html
+    #[cfg(dox)]
+    #[doc(cfg(target_arch = "mips"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub mod mips {}
+
+    /// Platform-specific intrinsics for the `mips64` platform.
+    ///
+    /// The documentation with the full listing of `mips64` intrinsics is
+    /// available in [libcore], but the module is re-exported here in std
+    /// as well.
+    ///
+    /// [libcore]: ../../../core/arch/mips64/index.html
+    #[cfg(dox)]
+    #[doc(cfg(target_arch = "mips64"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub mod mips64 {}
+
+    /// Platform-specific intrinsics for the `powerpc` platform.
+    ///
+    /// The documentation with the full listing of `powerpc` intrinsics is
+    /// available in [libcore], but the module is re-exported here in std
+    /// as well.
+    ///
+    /// [libcore]: ../../../core/arch/powerpc/index.html
+    #[cfg(dox)]
+    #[doc(cfg(target_arch = "powerpc"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub mod powerpc {}
+
+    /// Platform-specific intrinsics for the `powerpc64` platform.
+    ///
+    /// The documentation with the full listing of `powerpc64` intrinsics is
+    /// available in [libcore], but the module is re-exported here in std
+    /// as well.
+    ///
+    /// [libcore]: ../../../core/arch/powerpc64/index.html
+    #[cfg(dox)]
+    #[doc(cfg(target_arch = "powerpc64"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub mod powerpc64 {}
+
+    /// Platform-specific intrinsics for the `nvptx` platform.
+    ///
+    /// The documentation with the full listing of `nvptx` intrinsics is
+    /// available in [libcore], but the module is re-exported here in std
+    /// as well.
+    ///
+    /// [libcore]: ../../../core/arch/nvptx/index.html
+    #[cfg(dox)]
+    #[doc(cfg(any(target_arch = "nvptx", target_arch = "nvptx64")))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub mod nvptx {}
+
+    /// Platform-specific intrinsics for the `wasm32` platform.
+    ///
+    /// The documentation with the full listing of `wasm32` intrinsics is
+    /// available in [libcore], but the module is re-exported here in std
+    /// as well.
+    ///
+    /// [libcore]: ../../../core/arch/wasm32/index.html
+    #[cfg(dox)]
+    #[doc(cfg(target_arch = "wasm32"))]
+    #[unstable(feature = "stdsimd", issue = "27731")]
+    pub mod wasm32 {}
+}
